@@ -47,6 +47,7 @@ static zend_class_entry *swfeditor_ce;
 PHP_METHOD(swfed, __construct);
 PHP_METHOD(swfed, input);
 PHP_METHOD(swfed, output);
+//
 PHP_METHOD(swfed, getHeaderInfo);
 PHP_METHOD(swfed, setHeaderInfo);
 PHP_METHOD(swfed, getTagList);
@@ -54,12 +55,17 @@ PHP_METHOD(swfed, getTagDetail);
 PHP_METHOD(swfed, getTagInfo);
 PHP_METHOD(swfed, getTagData);
 PHP_METHOD(swfed, replaceTagData);
+//
+PHP_METHOD(swfed, setShapeAdjustMode);
+//
 PHP_METHOD(swfed, getJpegData);
 PHP_METHOD(swfed, getJpegAlpha);
 PHP_METHOD(swfed, replaceJpegData);
 PHP_METHOD(swfed, getPNGData);
 PHP_METHOD(swfed, replacePNGData);
 PHP_METHOD(swfed, replaceGIFData);
+PHP_METHOD(swfed, applyShapeMatrixFactor);
+PHP_METHOD(swfed, applyShapeRectFactor);
 PHP_METHOD(swfed, getSoundData);
 PHP_METHOD(swfed, replaceMLDData);
 PHP_METHOD(swfed, getEditString);
@@ -94,6 +100,10 @@ ZEND_END_MODULE_GLOBALS(swfed)
 #define SWFED_G(v) (swfed_globals.v)
 #endif
 
+#define REGISTER_SWFED_CLASS_CONST_LONG(const_name, value) \
+    zend_declare_class_constant_long(swfeditor_ce, const_name, sizeof(const_name)-1, (long)value TSRMLS_CC);
+
+
 #endif	/* PHP_SWFED_H */
 
 
