@@ -13,14 +13,18 @@
 #include "swf_tag_action.h"
 #include "swf_tag_jpeg.h"
 #include "swf_tag_lossless.h"
+#include "swf_tag_shape.h"
 #include "swf_action.h"
 #include "swf_object.h"
+#include "bitmap_util.h"
 
 swf_object_t *
 swf_object_open(void) {
     swf_object_t *swf;
     malloc_debug_start(); /* DEBUG XXX */
     swf = (swf_object_t *) calloc(sizeof(*swf), 1);
+    //
+    swf->shape_adjust_mode = 0;
     return swf;
 }
 
@@ -42,8 +46,9 @@ int
 swf_object_input(swf_object_t *swf, unsigned char *data,
                  unsigned long data_len) {
     int result;
-    bitstream_t *bs = bitstream_open();
-    swf_tag_t **tag;
+    bitstream_t *bs;
+    swf_tag_t *tag, *prev_tag, *head_tag;
+    bs = bitstream_open();
     bitstream_input(bs, data, data_len);
     result = swf_header_parse(bs, &swf->header);
     if (result) {
@@ -85,19 +90,35 @@ swf_object_input(swf_object_t *swf, unsigned char *data,
         bitstream_close(bs);
         return result;
     }
-    tag = &swf->tag;
+    head_tag = prev_tag = NULL;
     while(1) {
         long pos;
         pos = bitstream_getbytepos(bs);
         if ((pos == -1) || ((long) swf->header.file_length <= pos)) {
             break;
         }
-        *tag = swf_tag_create(bs);
-        if (tag == NULL) {
-            fprintf(stderr, "swf_object_input: swf_tag_create failed\n");
+        tag = swf_tag_create(bs);
+	if (tag == NULL) {
+	    swf_tag_t *next_tag;
+	    for (tag = head_tag ; tag ; tag = next_tag) {
+	        next_tag = tag->next;
+		swf_tag_destroy(tag);
+	    }
+	    bitstream_close(bs);
+	    return 1; // FAILURE
+	}
+        if (head_tag == NULL) {
+            head_tag = tag;
+        } else if (tag) {
+            prev_tag->next = tag;
+            tag->next = NULL;
         }
-        tag = &((*tag)->next);
+	if (tag->tag == 0) { // END Tag
+	  break; // SUCCESS
+	}
+        prev_tag = tag;
     }
+    swf->tag = head_tag;
     bitstream_close(bs);
     return 0;
 }
@@ -211,6 +232,86 @@ swf_object_get_tagdata(swf_object_t *swf, int tag_seqno,
     return NULL;
 }
 
+/* --- */
+
+swf_tag_t *
+swf_object_search_bitmap_tag(swf_object_t *swf, int bitmap_id) {
+    swf_tag_t *tag;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_search_bitmap_tag: swf == NULL\n");
+        return NULL;
+    }
+    for (tag=swf->tag ; tag ; tag=tag->next) {
+        register int tag_code = tag->tag;
+        if (isBitmapTag(tag_code)) {
+            if (swf_tag_identity(tag, bitmap_id) == 0) {
+                return tag; // match
+            }
+        }
+    }
+    return NULL;
+}
+
+/* --- */
+
+int
+swf_object_set_shape_adjust_mode(swf_object_t *swf, unsigned mode) {
+    if (swf == NULL) {
+        return 1;
+    }
+    swf->shape_adjust_mode = mode;
+    return 0;
+}
+
+int
+swf_object_adjust_shapebitmap(swf_object_t *swf, int bitmap_id,
+                              int old_width, int old_height,
+                              int new_width, int new_height) {
+    swf_tag_t *tag = NULL;
+    double width_scale = 0, height_scale = 0;
+    if (swf->shape_adjust_mode & SWFED_SHAPE_BITMAP_MATRIX_RESCALE) {
+        width_scale  = (double) old_width  / new_width;
+        height_scale = (double) old_height / new_height;
+        for (tag = swf->tag ; tag ; tag=tag->next) {
+            register int tag_code = tag->tag;
+            if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_identity(tag, bitmap_id) == 0)) {
+                swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
+
+                swf_tag_apply_shape_matrix_factor(tag, swf_tag_shape->shape_id,
+                                                  width_scale, height_scale,
+                                                  0, 0, 0, swf);
+            }
+        }
+    }
+    
+    if (swf->shape_adjust_mode & SWFED_SHAPE_BITMAP_RECT_RESIZE) {
+        width_scale  = (double) new_width  / old_width;
+        height_scale = (double) new_height / old_height;
+        for (tag = swf->tag ; tag ; tag=tag->next) {
+            register int tag_code = tag->tag;
+            if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_identity(tag, bitmap_id) == 0)) {
+                swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
+                swf_tag_apply_shape_rect_factor(tag, swf_tag_shape->shape_id,
+                                                width_scale, height_scale,
+                                                0, 0, swf);
+            }
+        }
+    }
+    if (swf->shape_adjust_mode & SWFED_SHAPE_BITMAP_TYPE_TILLED) {
+        for (tag = swf->tag ; tag ; tag=tag->next) {
+            register int tag_code = tag->tag;
+            if (isShapeTag(tag_code) && (swf_tag_shape_bitmap_identity(tag, bitmap_id) == 0)) {
+                swf_tag_shape_detail_t *swf_tag_shape = tag->detail;
+                swf_tag_apply_shape_type_tilled(tag, swf_tag_shape->shape_id, swf);
+            }
+        }
+    }
+}
+
+
+/* --- */
+
+
 int
 swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
                            unsigned char *data, unsigned long *length) {
@@ -218,8 +319,6 @@ swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
     return 1;
 }
 
-/* --- */
-
 unsigned char *
 swf_object_get_jpegdata(swf_object_t *swf, unsigned long *length, int image_id) {
     swf_tag_t *tag, *tag_jpegtables = NULL;
@@ -231,16 +330,10 @@ swf_object_get_jpegdata(swf_object_t *swf, unsigned long *length, int image_id)
     }
     for (tag=swf->tag ; tag ; tag=tag->next) {
         if (tag->tag == 8) { // JPEGTables
-            tag_jpegtables = tag;
-            break;
-        }
-    }
-    for (tag=swf->tag ; tag ; tag=tag->next) {
-        if (tag->tag == 8) {
             tag_jpegtables = tag;
             continue;
         }
-        // DefineBitsJPEG(1),2,3
+        // ! DefineBitsJPEG(1),2,3
         if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35)) {
             continue;
         }
@@ -282,17 +375,32 @@ swf_object_replace_jpegdata(swf_object_t *swf, int image_id,
                             unsigned long alpha_data_len) {
     int result = 1;
     swf_tag_t *tag;
+    int old_width, old_height, new_width, new_height;
     if (swf == NULL) {
         fprintf(stderr, "swf_object_replace_jpegdata: swf == NULL\n");
         return 1;
     }
-    for (tag=swf->tag ; tag ; tag=tag->next) {
-        result = swf_tag_replace_jpeg_data(tag, image_id,
-                                           jpeg_data, jpeg_data_len,
-                                           alpha_data, alpha_data_len);
-        if (! result) {
-            break;
-        }
+    tag = swf_object_search_bitmap_tag(swf, image_id);
+    if (tag == NULL) {
+        fprintf(stderr, "swf_object_replace_jpegdata: tag == NULL\n");
+        return 1;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_tag_create_input_detail(tag, swf);
+        swf_tag_get_bitmap_size(tag, &old_width, &old_height);
+        jpeg_size(jpeg_data, jpeg_data_len, &new_width, &new_height);
+    }
+    result = swf_tag_replace_jpeg_data(tag, image_id,
+                                       jpeg_data, jpeg_data_len,
+                                       alpha_data, alpha_data_len);
+    if (result) {
+        fprintf(stderr, "swf_object_replace_jpegdata: swf_tag_replace_jpeg_data failed\n");
+        return result;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_object_adjust_shapebitmap(swf, image_id,
+                                      old_width, old_height,
+                                      new_width, new_height);
     }
     return result;
 }
@@ -327,16 +435,31 @@ swf_object_replace_pngdata(swf_object_t *swf, int image_id,
                             unsigned long png_data_len) {
     int result = 1;
     swf_tag_t *tag;
+    int old_width, old_height, new_width, new_height;
     if (swf == NULL) {
         fprintf(stderr, "swf_object_replace_pngdata: swf == NULL\n");
         return 1;
     }
-    for (tag=swf->tag ; tag ; tag=tag->next) {
-        result = swf_tag_replace_png_data(tag, image_id,
-                                           png_data, png_data_len);
-        if (! result) {
-            break;
-        }
+    tag = swf_object_search_bitmap_tag(swf, image_id);
+    if (tag == NULL) {
+        fprintf(stderr, "swf_object_replace_pngdata: tag == NULL\n");
+        return 1;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_tag_create_input_detail(tag, swf);
+        swf_tag_get_bitmap_size(tag, &old_width, &old_height);
+        png_size(png_data, png_data_len, &new_width, &new_height);
+    }
+    result = swf_tag_replace_png_data(tag, image_id,
+                                      png_data, png_data_len);
+    if (result) {
+        fprintf(stderr, "swf_object_replace_pngdata: swf_tag_replace_png_data failed\n");
+        return result;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_object_adjust_shapebitmap(swf, image_id,
+                                      old_width, old_height,
+                                      new_width, new_height);
     }
     return result;
 }
@@ -352,16 +475,31 @@ swf_object_replace_gifdata(swf_object_t *swf, int image_id,
                             unsigned long gif_data_len) {
     int result = 1;
     swf_tag_t *tag;
+    int old_width, old_height, new_width, new_height;
     if (swf == NULL) {
         fprintf(stderr, "swf_object_replace_gifdata: swf == NULL\n");
         return 1;
     }
-    for (tag=swf->tag ; tag ; tag=tag->next) {
-        result = swf_tag_replace_gif_data(tag, image_id,
-                                          gif_data, gif_data_len);
-        if (! result) {
-            break;
-        }
+    tag = swf_object_search_bitmap_tag(swf, image_id);
+    if (tag == NULL) {
+        fprintf(stderr, "swf_object_replace_gifdata: tag == NULL\n");
+        return 1;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_tag_create_input_detail(tag, swf);
+        swf_tag_get_bitmap_size(tag, &old_width, &old_height);
+        gif_size(gif_data, gif_data_len, &new_width, &new_height);
+    }
+    result = swf_tag_replace_gif_data(tag, image_id,
+                                      gif_data, gif_data_len);
+    if (result) {
+        fprintf(stderr, "swf_object_replace_pngdata: swf_tag_replace_png_data failed\n");
+        return result;
+    }
+    if (swf->shape_adjust_mode) {
+        swf_object_adjust_shapebitmap(swf, image_id,
+                                      old_width, old_height,
+                                      new_width, new_height);
     }
     return result;
 }
@@ -476,3 +614,66 @@ swf_object_get_actiondata(swf_object_t *swf, unsigned long *length, int tag_seqn
     *length = swf_tag_action->action_record_len;
     return swf_tag_action->action_record;
 }
+
+int
+swf_object_apply_shapematrix_factor(swf_object_t *swf, int shape_id,
+                                    double scale_x, double scale_y,
+                                    double rotate_rad,
+                                    signed int trans_x, signed int trans_y) {
+    int result = 1;
+    swf_tag_t *tag;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_apply_shapematrix_factor: swf == NULL\n");
+        return 1;
+    }
+    for (tag=swf->tag ; tag ; tag=tag->next) {
+        result = swf_tag_apply_shape_matrix_factor(tag, shape_id,
+                                                   scale_x, scale_y,
+                                                   rotate_rad,
+                                                   trans_x, trans_y,
+                                                   swf);
+        if (! result) {
+            break;
+        }
+    }
+    return result;
+}
+
+int
+swf_object_apply_shaperect_factor(swf_object_t *swf, int shape_id,
+                                 double scale_x, double scale_y,
+                                 signed int trans_x, signed int trans_y) {
+    int result = 1;
+    swf_tag_t *tag;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_apply_shaperect_factor: swf == NULL\n");
+        return 1;
+    }
+    for (tag=swf->tag ; tag ; tag=tag->next) {
+        result = swf_tag_apply_shape_rect_factor(tag, shape_id,
+                                                   scale_x, scale_y,
+                                                   trans_x, trans_y,
+                                                   swf);
+        if (! result) {
+            break;
+        }
+    }
+    return result;
+}
+
+int
+swf_object_apply_shapetype_tilled(swf_object_t *swf,int shape_id) {
+    int result = 1;
+    swf_tag_t *tag;
+    if (swf == NULL) {
+        fprintf(stderr, "swf_object_apply_shaperect_factor: swf == NULL\n");
+        return 1;
+    }
+    for (tag=swf->tag ; tag ; tag=tag->next) {
+        result = swf_tag_apply_shape_type_tilled(tag, shape_id, swf);
+        if (! result) {
+            break;
+        }
+    }
+    return result;
+}
