@@ -15,8 +15,13 @@ typedef struct swf_object_ {
     swf_header_t header;
     swf_header_movie_t header_movie;
     swf_tag_t *tag;
+    unsigned shape_adjust_mode;
 } swf_object_t;
 
+#define SWFED_SHAPE_BITMAP_NONE           0
+#define SWFED_SHAPE_BITMAP_MATRIX_RESCALE 1
+#define SWFED_SHAPE_BITMAP_RECT_RESIZE    2
+#define SWFED_SHAPE_BITMAP_TYPE_TILLED    4
 
 extern swf_object_t *swf_object_open(void);
 extern void swf_object_close(swf_object_t *swf);
@@ -34,6 +39,18 @@ extern int swf_object_replace_tagdata(swf_object_t *swf, int tag_seqno,
 
 /* --- */
 
+extern swf_tag_t *swf_object_search_bitmap_tag(swf_object_t *swf,
+                                               int bitmap_id);
+
+/* --- */
+
+extern int swf_object_set_shape_adjust_mode(swf_object_t *swf, unsigned mode);
+extern int swf_object_adjust_shapebitmap(swf_object_t *swf, int bitmap_id,
+                                         int old_width, int old_height,
+                                         int new_width, int new_height);
+
+/* --- */
+
 extern unsigned char *swf_object_get_jpegdata(swf_object_t *swf, unsigned long *length, int image_id);
 extern unsigned char *swf_object_get_alphadata(swf_object_t *swf, unsigned long *length, int image_id);
 extern int swf_object_replace_jpegdata(swf_object_t *swf, int image_id,
@@ -62,4 +79,15 @@ extern int swf_object_replace_editstring(swf_object_t *swf,
                                          int initial_text_len);
 extern unsigned char *swf_object_get_actiondata(swf_object_t *swf, unsigned long *length, int tag_seqno);
 
+extern int swf_object_apply_shapematrix_factor(swf_object_t *swf,int shape_id,
+                                               double scale_x, double scale_y,
+                                               double rotate_rad,
+                                               signed int trans_x,
+                                               signed int trans_y);
+extern int swf_object_apply_shaperect_factor(swf_object_t *swf,int shape_id,
+                                             double scale_x, double scale_y,
+                                             signed int trans_x,
+                                             signed int trans_y);
+extern int swf_object_apply_shapetype_tilled(swf_object_t *swf,int shape_id);
+
 #endif /* __SWF_OBJECT_H__ */
