@@ -16,18 +16,18 @@
 #include "swf_tag_sound.h"
 #include "swf_tag_sprite.h"
 #include "swf_tag_shape.h"
+#include "bitmap_util.h"
 
 swf_tag_info_t swf_tag_info_table[] = {
     { 0, "End", NULL },
     { 1, "ShowFrame", NULL },
-    //    { 2, "DefineShape", swf_tag_shape_detail_handler },
-    { 2, "DefineShape", NULL },
+    { 2, "DefineShape", swf_tag_shape_detail_handler },
     { 3, "FreeCharacter", NULL},
     { 4, "PlaceObject", NULL},
     { 5, "RemoveObject", NULL},
     { 6, "DefineBitsJPEG", swf_tag_jpeg_detail_handler },
     { 7, "DefineButton", NULL},
-    { 8, "JPEGTables", NULL },
+    { 8, "JPEGTables", swf_tag_jpegt_detail_handler },
     { 9, "SetBackgroundColor", NULL },
     { 10, "DefineFont ", NULL},
     { 11, "DefineText", NULL },
@@ -35,18 +35,16 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 13, "DefineFontInfo", NULL },
     { 14, "DefineSound", swf_tag_sound_detail_handler },
     { 15, "StartSound", NULL },
-    { 16, "DefineButtonSound", NULL },
+    // 16 missing
     { 17, "DefineButtonSound", NULL },
     { 18, "SoundStreamHead", NULL },
     { 19, "SoundStreamBlock", NULL },
     { 20, "DefineBitsLossless", swf_tag_lossless_detail_handler },
     { 21, "DefineBitsJPEG2", swf_tag_jpeg_detail_handler },
-    //    { 22, "DefineShape2", swf_tag_shape_detail_handler },
-    { 22, "DefineShape2", NULL },
+    { 22, "DefineShape2", swf_tag_shape_detail_handler },
     { 26, "PlaceObject2", NULL },
     { 28, "RemoveObject2", NULL },
-    //    { 32, "DefineShape3", swf_tag_shape_detail_handler },
-    { 32, "DefineShape3", NULL },
+    { 32, "DefineShape3", swf_tag_shape_detail_handler },
     { 33, "DefineText2", NULL },
     { 34, "DefineButton2", NULL },
     { 35, "DefineBitsJPEG3", swf_tag_jpeg3_detail_handler },
@@ -54,8 +52,7 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 37, "DefineEditText", swf_tag_edit_detail_handler },
     { 39, "DefineSprite", swf_tag_sprite_detail_handler },
     { 43, "FrameLabel", NULL } ,
-    //    { 46, "DefineMorphShape", swf_tag_shape_detail_handler },
-    { 46, "DefineMorphShape", NULL },
+    { 46, "DefineMorphShape", swf_tag_shape_detail_handler },
     { 48, "DefineFont2", NULL } ,
     { 56, "Export", NULL } ,
     { 59, "DoInitAction", NULL } ,
@@ -63,10 +60,8 @@ swf_tag_info_t swf_tag_info_table[] = {
     { 73, "DefineFontAlignZones", NULL },
     { 74, "CSMTextSettings", NULL },
     { 75, "DefineFont3", NULL } ,
-    //    { 83, "DefineShape4", swf_tag_shape_detail_handler },
-    { 83, "DefineShape4", NULL },
-    //    { 84, "DefineMorphShape2", swf_tag_shape_detail_handler },
-    { 84, "DefineMorphShape2", NULL},
+    { 83, "DefineShape4", NULL /*swf_tag_shape_detail_handler*/ },
+    { 84, "DefineMorphShape2", NULL /*swf_tag_shape_detail_handler*/ },
     { 88, "DefineFontName", NULL } ,
     { 777,"Reflex", NULL } ,
 };
@@ -90,6 +85,7 @@ swf_tag_t *swf_tag_create(bitstream_t *bs) {
     }
     tag_and_length = bitstream_getbytesLE(bs, 2);
     if (tag_and_length == (unsigned short) -1) {
+        fprintf(stderr, "swf_tag_create: tag_and_length(short) == -1\n");
         free(tag);
         return NULL;
     }
@@ -99,6 +95,7 @@ swf_tag_t *swf_tag_create(bitstream_t *bs) {
     if (tag->length == 0x3f) {
         tag->length = bitstream_getbytesLE(bs, 4);
         if (tag_and_length == (unsigned short) -1) {
+	    fprintf(stderr, "swf_tag_create: tag_and_length(long) == -1\n");
             free(tag);
             return NULL;
         }
@@ -135,6 +132,7 @@ void swf_tag_destroy(swf_tag_t *tag) {
     free(tag);
 }
 
+
 static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
     signed short tag_and_length;
     if (bs == NULL) {
@@ -168,7 +166,8 @@ static int swf_tag_and_length_build(bitstream_t *bs, swf_tag_t *tag) {
     }
     return 0;
 }
-extern int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf) {
+
+int swf_tag_build(bitstream_t *bs, swf_tag_t *tag, struct swf_object_ *swf) {
     swf_tag_info_t *tag_info;
     unsigned char *data;
     unsigned long data_len = 0;
@@ -266,6 +265,52 @@ int swf_tag_create_input_detail(swf_tag_t *tag, struct swf_object_ *swf) {
     return 1;
 }
 
+int
+swf_tag_identity(swf_tag_t *tag, int cid) {
+    swf_tag_info_t *tag_info;
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->identity) {
+            if (detail_handler->identity(tag, cid)) {
+                return 1; // no match
+            } else{
+                return 0; // match
+            }
+        }
+    }
+    return 1; // no match - no identity method
+}
+
+/* bitmap */
+
+int
+swf_tag_get_bitmap_size(swf_tag_t *tag,
+                        int *width, int *height) {
+    int ret = 0;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_get_bitmap_size: tag == NULL\n");
+        return 1;
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_get_bitmap_size: tag->detail == NULL\n");
+        return 1;
+    }
+
+    if (isBitsJPEGTag(tag->tag)) {
+        swf_tag_jpeg_detail_t *swf_tag_jpeg = (swf_tag_jpeg_detail_t *) tag->detail;
+        ret = jpeg_size(swf_tag_jpeg->jpeg_data, swf_tag_jpeg->jpeg_data_len,
+                  width, height);
+    } else if (isBitsLosslessTag(tag->tag)) {
+        swf_tag_lossless_detail_t *swf_tag_lossless = (swf_tag_lossless_detail_t *) tag->detail;
+        *width  = swf_tag_lossless->width;
+        *height = swf_tag_lossless->height;
+    }
+    return ret;
+
+}
+
+
 unsigned char *
 swf_tag_get_jpeg_data(swf_tag_t *tag, unsigned long *length, int image_id, swf_tag_t *tag_jpegtables) {
     swf_tag_info_t *tag_info;
@@ -338,10 +383,7 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_jpeg_data: tag == NULL\n");
         return 1;
     }
-    // DefineBitsJPEG or 2 or 3
-    // BitsLossless or 2
-    if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35) &&
-        (tag->tag != 20) && (tag->tag != 36)) {
+    if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag);
@@ -349,10 +391,12 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
     if (detail_handler->identity(tag, image_id)) {
         return 1;
     }
+    /*
     if (tag->detail) {
         detail_handler->destroy(tag);
         tag->detail = NULL;
     }
+    */
     if (alpha_data && (alpha_data_len > 0)) {
         tag->tag = 35;
     } else {
@@ -360,9 +404,11 @@ swf_tag_replace_jpeg_data(swf_tag_t *tag, int image_id,
             tag->tag = 21;
         }
     }
-    tag_info = get_swf_tag_info(tag->tag);
-    detail_handler = tag_info->detail_handler();
-    tag->detail = detail_handler->create();
+
+    if (tag->detail == NULL) {
+        tag->detail = detail_handler->create();
+    }
+    
     result= swf_tag_jpeg_replace_jpeg_data(tag->detail, image_id,
                                            jpeg_data, jpeg_data_len,
                                            alpha_data, alpha_data_len, tag);
@@ -413,10 +459,7 @@ swf_tag_replace_png_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_png_data: tag == NULL\n");
         return 1;
     }
-    // DefineBitsJPEG or 2 or 3
-    // BitsLossless or 2
-    if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35) &&
-        (tag->tag != 20) && (tag->tag != 36)) {
+    if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag);
@@ -465,10 +508,7 @@ swf_tag_replace_gif_data(swf_tag_t *tag, int image_id,
         fprintf(stderr, "swf_tag_replace_gif_data: tag == NULL\n");
         return 1;
     }
-    // DefineBitsJPEG or 2 or 3
-    // BitsLossless or 2
-    if ((tag->tag != 6) && (tag->tag != 21) && (tag->tag != 35) &&
-        (tag->tag != 20) && (tag->tag != 36)) {
+    if (! isBitmapTag(tag->tag)) {
         return 1;
     }
     tag_info = get_swf_tag_info(tag->tag);
@@ -623,3 +663,132 @@ swf_tag_replace_edit_string(swf_tag_t *tag,
     }
     return result;
 }
+
+
+int
+swf_tag_apply_shape_matrix_factor(swf_tag_t *tag, int shape_id,
+                                  double scale_x, double scale_y,
+                                  double rotate_rad,
+                                  signed int trans_x, signed int trans_y,
+                                  struct swf_object_ *swf) {
+    swf_tag_info_t *tag_info;
+    int result;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_matrix_factor: tag == NULL\n");
+        return 1;
+    }
+    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
+        && (tag->tag != 46)) {
+        // ! DefineShape1,2,3, DefineMorphShape
+        return 1;
+    }
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->identity) {
+            if (detail_handler->identity(tag, shape_id)) {
+                return 1;
+            }
+        }
+    }
+    if (! tag->detail) {
+        swf_tag_create_input_detail(tag, swf);
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_apply_shape_matrix_factor: Can't create tag\n");
+        return 1;
+    }
+    result = swf_tag_shape_apply_matrix_factor(tag->detail, shape_id,
+                                                     scale_x, scale_y,
+                                                     rotate_rad,
+                                                     trans_x, trans_y);
+    if (result == 0) {
+        free(tag->data);
+        tag->data = NULL;
+        tag->length = 0;
+    }
+    return result;
+}
+
+int
+swf_tag_apply_shape_rect_factor(swf_tag_t *tag, int shape_id,
+                                  double scale_x, double scale_y,
+                                  signed int trans_x, signed int trans_y,
+                                  struct swf_object_ *swf) {
+    swf_tag_info_t *tag_info;
+    int result;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_rect_factor: tag == NULL\n");
+        return 1;
+    }
+    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
+        && (tag->tag != 46)) {
+        // ! DefineShape1,2,3, DefineMorphShape
+        return 1;
+    }
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->identity) {
+            if (detail_handler->identity(tag, shape_id)) {
+                return 1;
+            }
+        }
+    }
+    if (! tag->detail) {
+        swf_tag_create_input_detail(tag, swf);
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_apply_shape_rect_factor: Can't create tag\n");
+        return 1;
+    }
+    result = swf_tag_shape_apply_rect_factor(tag->detail, shape_id,
+                                             scale_x, scale_y,
+                                             trans_x, trans_y);
+    if (result == 0) {
+        free(tag->data);
+        tag->data = NULL;
+        tag->length = 0;
+    }
+    return result;
+}
+
+int
+swf_tag_apply_shape_type_tilled(swf_tag_t *tag, int shape_id,
+                                struct swf_object_ *swf) {
+    swf_tag_info_t *tag_info;
+    int result;
+    if (tag == NULL) {
+        fprintf(stderr, "swf_tag_apply_shape_type_tylled: tag == NULL\n");
+        return 1;
+    }
+    if ((tag->tag != 2) && (tag->tag != 22) && (tag->tag !=32 )
+        && (tag->tag != 46)) {
+        // ! DefineShape1,2,3, DefineMorphShape
+        return 1;
+    }
+    tag_info = get_swf_tag_info(tag->tag);
+    if (tag_info && tag_info->detail_handler) {
+        swf_tag_detail_handler_t * detail_handler = tag_info->detail_handler();
+        if (detail_handler->identity) {
+            if (detail_handler->identity(tag, shape_id)) {
+                return 1;
+            }
+        }
+    }
+    if (! tag->detail) {
+        swf_tag_create_input_detail(tag, swf);
+    }
+    if (! tag->detail) {
+        fprintf(stderr, "swf_tag_apply_shape_rect_factor: Can't create tag\n");
+        return 1;
+    }
+    result = swf_tag_shape_apply_type_tilled(tag->detail, shape_id);
+    if (result == 0) {
+        free(tag->data);
+        tag->data = NULL;
+        tag->length = 0;
+    }
+    return result;
+    
+}
